#include "common.h"
#include "task.h"
#include "fifo.h"
#include "event.h"
#include "lowio.h"
#include "klib.h"
#include "mem.h"
#include "pen.h"
#include "vt.h"
#include "keyboard.h"
#include "lcd.h"
#include "file.h"
#include "flash.h"
#include "file_action.h"
#include "slip.h"

extern void eventd();
extern void test_lissa();
extern void test_4bpp();
extern void test_2bpp();

extern TSS task_table[];
extern short ntask;
extern vt vt_table[];
extern unsigned char u_ram_map[];
extern unsigned char s_ram_map[];

extern short calibrando;

void gettapxy(vt *pvt,int *x, int *y)
{
	int ev;
	cursor_off(pvt);
	do {
		ev=fifo_get(pvt->event);
		switch(ev) {
			case EV_KEYBOARD:
				//kputchar(fifo_get(pvt->event));
				fifo_get(pvt->event);
				break;
			case EV_LCD_TAP_PRESS:
				*x=fifo_get(pvt->event);
				*y=fifo_get(pvt->event);
				break;
			case EV_LCD_TAP_REL:
				break;
			case EV_FOCUS_ON:
				break;
			case EV_FOCUS_OFF:
				break;
		}
	}while (ev!=EV_LCD_TAP_PRESS);
}

int gettap(vt *pvt)
{
	int x,y;
	gettapxy(pvt,&x,&y);
	return y>>3;
}

int si_no(vt *pvt)
{
	int x,y;
	gotoxy(0,11);
	kprintf("     SI      NO");
	gettapxy(pvt,&x,&y);
	y>>=3;
	x>>=5;
	if(y==11 && x==1) return 1;
	else return 0;
}

/*----------------------------------------------------------------------------*/
void nyi(vt *pvt)
{
	cls();
	gotoxy(0,9);  kprintf("An no implementado\n");
	gotoxy(0,19); kprintf("<<");
	gettap(pvt);
}

/*---------------------------------------------------------------------------*/
void show_batt(vt *pvt)
{
	int v,i,x,y,cnt,c,oc;
	cls();

	kprintf("ESTADO de las PILAS");
	
	hline(0,159,11,1);
	gotoxy(0,19); kprintf("<<");
	hline(0,159,159-11,1);
	oc=2;
	rectangle(0,80,159,90,1);
	
	v=get_batt();
	for(cnt=0;;){
		if (!cnt) {
		  i=v/1000;
		  gotoxy(0,6); kprintf("Vbat = %01d.%02d Voltios",i,(v-i*1000)/10);
		  gotoxy(0,8); kprintf("Capacidad = %2d %%",(v-2200)/10);
		  cursor_off(pvt);
		  c=(v-2200)*159/1000;
		  if (c>oc) filled_rectangle(oc,82,c,88,1);
		  else filled_rectangle(c,82,oc,88,0);
		  oc=c;
		}
		cnt++; if (cnt==16) cnt=0;
		v=(v*(256-2)+2*get_batt())>>8;
		task_sleep(1);
		i=fifo_getnb(pvt->event);
		if (i==EV_KEYBOARD) x=fifo_get(pvt->event);
		if (i==EV_LCD_TAP_PRESS) {
			x=fifo_get(pvt->event);
			y=fifo_get(pvt->event);
			y=(y+4)>>4;
			if (y>=9) return;
		}
	}
}

/*---------------------------------------------------------------------------*/

extern unsigned int slip_ndrop,slip_nrx,slip_ntx;
volatile extern short int txst,rxst;
extern slip_pkt pkt_pool[];
extern slip_pkt rx_q[],tx_q[];
volatile extern slip_pkt *rx_wp,*rx_rp,*tx_wp,*tx_rp;

void slip_st(vt *pvt)
{
	int ev,i,j,k;
	for(;;) {
		cls();
		kprintf("SLIP STATUS\n\n");
		hline(0,159,11,1);
		kprintf("slip_ndrop = %d\n",slip_ndrop);
		kprintf("slip_nrx   = %d\n",slip_nrx); 
		kprintf("slip_ntx   = %d\n\n",slip_ntx); 
		kprintf("RX_STATUS  = %d\n",rxst);
		kprintf("TX_STATUS  = %d\n\n",txst);
		kprintf("buffers : ");
		for (i=0;i<NBUF;i++) kputchar((pkt_pool[i].len)?'U':'_');

		j=((int)rx_wp-(int)rx_q)>>3; k=((int)rx_rp-(int)rx_q)>>3;
		kprintf("\n\nCola RX pw=%d pr=%d\n",j,k);
		j=((int)tx_wp-(int)tx_q)>>3; k=((int)tx_rp-(int)tx_q)>>3;
		kprintf("Cola TX pw=%d pr=%d",j,k);
				
		for(i=0;i<10;i++) {
			ev=fifo_getnb(pvt->event);
			if (ev!=-1) break;
			task_sleep(6);
		}
		switch(ev) {
			case EV_KEYBOARD:
				fifo_get(pvt->event);
				break;
			case EV_LCD_TAP_PRESS:
				fifo_get(pvt->event);
				fifo_get(pvt->event);
				return;
			}
	}
}

/*----------------------------------------------------------------------------*/

void show_mem(vt *pvt,int tix)
{
	int i,j,c,fu,fs,mt,x,y;
	
	cls();
	kprintf("ESTADO de la MEMORIA");
	hline(0,159,11,1);
	rectangle(15,32,144,65,1);
	rectangle(15,11*8,144,11*8+33,1);
	hline(0,159,159-11,1);
	gotoxy(0,19); kprintf("<<");
	
	if (tix==0) tix=255;
	for (;;) {
	  fu=fs=mt=0;
	  for (i=0;i<16;i++)
	     for(j=0;j<64;j++) {
	     	if (u_ram_map[(i<<6)+j]) c=1; else { c=0; fu++;}
	     	x=j*2+16; y=i*2+33;
	     	putpixel(x,y,c); putpixel(x+1,y,c);
	     	putpixel(x,y+1,c); putpixel(x+1,y+1,c);
	     	if (u_ram_map[(i<<6)+j]==tix) {
		  putpixel(x,y,0); putpixel(x+1,y+1,0);
	     	  mt++;
	     	}
	     	if (s_ram_map[(i<<6)+j]) c=1; else {c=0; fs++;}
	     	x=j*2+16; y=i*2+11*8+1;
	     	putpixel(x,y,c); putpixel(x+1,y,c);
	     	putpixel(x,y+1,c); putpixel(x+1,y+1,c);
	     	if (s_ram_map[(i<<6)+j]==tix) {
	     	  putpixel(x,y,0); putpixel(x+1,y+1,0);
	     	  mt++;
	     	}
	     }
	  gotoxy(0,3);  kprintf("USER  %4d Kb libres",fu>>(10-L2PAGE));
	  gotoxy(0,10); kprintf("SUPER %4d Kb libres",fs>>(10-L2PAGE));
	  if (tix>=0) {
	    gotoxy(0,17);
	    kprintf("TAREA: %d bytes",mt<<L2PAGE);
	  }
	  cursor_off(pvt);
	  i=gettap(pvt);
	  if (i>=18) return;
	} 
}

/*---------------------------------------------------------------------------*/
extern unsigned long long system_clks;

void tasker(vt *pvt)
{
	short tareas[MAXTASK];
	int i,j,pri,nt,nvt,ev,x,y,sel;
	char s,*p;
	unsigned long long osys,oidle;
	
	sel=-1;
	for(;;) {
	  cls();
	  kprintf("TAREAS  CPU\n\nID PRI ST VT Nombre\n\n");
	  hline(0,159,11,1);
	  hline(0,159,27,1);
	  
	  // ***** CPU Load ****
	  i=system_clks-osys; osys=system_clks;
	  j=task_table[0].clks-oidle; oidle=task_table[0].clks;
	  i>>=6; j/=i;
	  if(j<0) j=0;
	  if(j>63) j=63;
	  rectangle(92,0,159,7,1);
	  filled_rectangle(94,2,157-j,5,1);
	  
	  for (i=nt=x=0;i<ntask;i++) {
		if (task_table[i].status>KILLED) tareas[nt++]=i;
	  }
	  if (sel>=nt) sel=-1;
	  for (i=0;i<nt;i++) {
		j=task_table[tareas[i]].status;
		if (j>0) s='T';
		else if (j==0) s='R';
		else s='S';
		kprintf("%2d %3d %c ",	tareas[i],task_table[tareas[i]].priority,s);
		if (task_table[tareas[i]].vt!=(vt *)0) {
		  nvt=((int)task_table[tareas[i]].vt-(int)vt_table)/sizeof(vt);
		  kprintf("%2d ",nvt);
		} else kprintf("   ");
		kprintf("%s\n",task_table[tareas[i]].name);
		
	  }

	  if (sel>=0) {
		p=pvt->screen_base;
		i=(sel+4)*8*20; j=i+8*20;
		for (;i<j;i++) p[i]^=0xff;
	  }
	
	  hline (0,159,159-11,1);
	  gotoxy(0,19); kprintf("<<   KILL GOTO MEM ");
	  cursor_off(pvt);	
	  for(;;){
	  	for (i=16;i;i--) {
			ev=fifo_getnb(pvt->event);
			if (ev!=-1) break;
			task_sleep(4);
		}
		if (ev==-1) break;	//redibujamos cada segundo
		if (ev==EV_FOCUS_ON) break;
		if (ev==EV_KEYBOARD) {
			i=fifo_get(pvt->event);
			if (sel>2) {
			  pri=task_table[tareas[sel]].priority;
			  if (i==K_U) pri++;
			  if (i==K_D) pri--;
			  if (i==K_R) pri+=10; 
			  if (i==K_L) pri-=10;
			  if (pri<1) pri=1; if (pri>99) pri=99;
			  task_table[tareas[sel]].priority=pri;
			}
			break;
		}
		if (ev==EV_LCD_TAP_PRESS) {
			x=fifo_get(pvt->event);
			y=fifo_get(pvt->event);
			if (y>19*8) {
				i=x/40;
				if (i==0) return;	// Principal
				else if (i==3) {	// Memoria
				  if (sel>=0) show_mem(pvt,tareas[sel]);
				  else show_mem(pvt,-1);
				  break;
				}else if (i==2) {	// GoTO
				  if (sel>=0) {
				    if (task_table[tareas[sel]].vt!=(vt *)0) {
				    	switch_vt(task_table[tareas[sel]].vt);
				    	break;
				    }
				  }
				} else if (i==1) {	// Kill
				  if (sel>=0)
				     if (tareas[sel]>3)
				        task_kill(tareas[sel]);
				  sel=-1;
				  break;
				}
			} else {
				i=y/8-4;
				if (i>=0 && i<nt) {
					sel=i;
				}
				break;
			}
		}
	  }
	}
}

/*--------------------------------------------------------------------------*/

void aplicaciones(vt *pvt)
{
	int i,n;
	app *tabla_app=(app *)0x10c2000c;
	
	do {
	  cls();
	  kprintf("APLICACIONES\n\n");
	  hline(0,159,11,1);
	  i=0;
	  while (tabla_app[i].entry) {
	  	kprintf("%s\n",tabla_app[i++].name);
	  }
	  n=i;
	  gotoxy(0,19); kprintf("<<");
	  hline(0,159,159-11,1);
	  i=gettap(pvt)-2;
	  if (i>=0 && i<n) 
	    task_start(tabla_app[i].entry,0x0000,tabla_app[i].stack_size,
	               tabla_app[i].priority,tabla_app[i].name);
	} while (i<18-2);
}

/*----------------------------------------------------------------------------*/
extern const char *file_type[];
extern const char *long_file_type[];
extern const char *bin_code_type[];
extern const char *txt_type[];
extern const char *gr_type[];

void file_defrag();

void filer(vt *pvt) {
    int i,j,ix=-1,x,y,n,tipo,subt;
    file_header *pf;
    char *p;
    
    for(;;) {
	cls();
	kprintf("Directorio\n\n");
	if (ix==-1) {
		hline(0,159,11,1);
		for(i=1,n=0;i<NFTYPES;i++) {
			j=file_get_nfiles(i,0);
			n+=j;
			kprintf("%s ",file_type[i]);
			if(j) kprintf("%4d ficheros",j);
			kprintf("\n");
		}
		kprintf("TOT %4d ficheros\n",n);
	
		gotoxy(0,16);
		i=file_get_free(0);
		kprintf("FS0 %4d/%d Kb\n",i>>10,(i+file_get_del(0))>>10);
#ifdef DV2MB
		i=file_get_free(1);
		kprintf("FS1 %4d/%d Kb\n",i>>10,(i+file_get_del(1))>>10);
#endif
		hline(0,159,159-11-24,1);
		hline(0,159,159-11,1);
		gotoxy(0,19); kprintf("<<      DFRG");
		gettapxy(pvt,&x,&y); i=y>>3;
		if(i==19) switch(x>>5) {
			case 0:	return;
			case 2: file_defrag(); break;
		}
		tipo=i-1; if (tipo==NFTYPES) tipo=0;
		if(tipo>NFTYPES) continue;
	}
	if (ix==-1) ix=0;
	if((ix=file_sel_int(tipo, 0, ix))==-1) continue;
	pf=file_sel_ix(tipo,0,ix);
	cls();
	kprintf("Fichero\n\n");
	hline(0,159,11,1);
	hline(0,159,159-11,1);
	kprintf("%s\n\n",pf->name);
	kprintf("Tipo:%s\n",long_file_type[pf->type]);
	p="?";
	if(pf->type==BINARY_CODE) p=(char *)bin_code_type[pf->subtype];
	else if(pf->type==TEXT) p=(char *)txt_type[pf->subtype];
	else if(pf->type==GRAPHIC) p=(char *)gr_type[pf->subtype]; 
	kprintf("Subt:%s\n",p);
	kprintf("Size:%d\n",pf->size);
	kprintf("Addr:%x\n",(int)pf+sizeof(file_header));
	gotoxy(0,19); kprintf("<<  VER  DEL ");
	do gettapxy(pvt,&i,&j);
	while ((j>>3)<19);
	switch((i>>5)) {
	case 1:	file_take_action(pf);
		break;
	case 2:	// delete
		cls();
		gotoxy(0,7); kprintf("Borrar Fichero:\n\n%s?",pf->name);
		if(si_no(pvt)) {
		    flash_prg_w((long)&(pf->type),0);
		    cls(); gotoxy(0,9); kprintf("%s\n\n BORRADO",pf->name);
		    gettap(pvt);
		}
		break;
	}
    }	
}

/*****************************************************************************/

void remoted();

void shell()
{
	vt *pvt;
	int i,j;

	pvt=open_vt(0);
	switch_vt(pvt);
	cls();

	for (i=0;i<160;i+=4) {
		line(0,i,i,159,1);
		line(i,0,159,i,1);
	}
	line(0,159,159,159,1);
	line(159,0,159,159,1);

	for (i=30;i<54;i+=2) circle(80,80,i,1);
	gotoxy(8,8); kprintf("FEOS");
	gotoxy(8,11);kprintf("v2.2\n");

	cursor_off((vt *)0);
	
	file_init();
	task_start(remoted,0x2000,512,50,"remoted");
	gettap(pvt);
	cls();
	

//	Menu principal:

	for(;;) {
		cls();
		kprintf("PRINCIPAL\n\n");
		hline(0,159,11,1);
		kprintf("UTILIDADES\n\n");
		kprintf("APLICACIONES\n\n");
		kprintf("FICHEROS\n\n");
		kprintf("TAREAS\n\n");
		kprintf("MEMORIA\n\n");

		i=gettap(pvt);
		switch(i) {
		case 2:	// utilidades
			do {
			  cls();
			  kprintf("UTILIDADES\n\n");
			  hline(0,159,11,1);
			  kprintf("HORA\n\n");
			  kprintf("PILAS\n\n");
			  kprintf("ESTADISTICAS SLIP\n\n");
			  kprintf("CALIBRADO\n\n");
			  kprintf("BOOTLOADER SERIE\n\n");
			  kprintf("LUZ ON/OFF");
			  gotoxy(0,19); kprintf("<<");
			  hline(0,159,159-11,1);
			  i=gettap(pvt);
			  switch(i) {
			  case 2: // hora
			  	  nyi(pvt);
				  break;
			  case 4: // Pilas
			  	  show_batt(pvt); break; 
			  case 6: // Estado de comunicaciones
			  	  slip_st(pvt); break;
			  case 8: // calibrado
			  	  calibrando=1; break;
				  //pen_calibrate(); break;
			  case 10:  //Bootloader serie
			  	  cls();
			  	  gotoxy(2,9); kprintf("Bootloader Serie");
		        	  asm volatile("
		        	    move.w	#0x2700,%sr
		                    move.l	#0xffffffff,0xfffff304
		                    move.w	#0x2420,0xfffff200
		                    jmp		0xffffff00
				  ");
			  case 12:  // luz
			  	  *(char *)0xfffff409^=0x40;
			  	  break;
			  }
			} while (i<18);
			
			break;
		case 4: // aplicaciones
			aplicaciones(pvt);
			break;
		case 6: // ficheros
			filer(pvt);
			break;
		case 8: // tareas
			tasker(pvt);
			break;
		case 10: // memoria
			show_mem(pvt,-1);
			break;
		}
	}
}
